#ifndef _fencer_state_h
#define _fencer_state_h

#include "typeclass_helpers.h"

typedef struct State State;

struct State {
    void (*const enter)(void* data);
    void (*const exit)(void* data);
    void (*const update)(void* data, float dt);
    const State* (*const next)(void* data);
};

#define DefineState(_StateName, enter_fn, exit_fn, update_fn, next_fn)\
static inline const State* _StateName() {\
    TC_FN_TYPECHECK(void, enter_fn, void*);\
    TC_FN_TYPECHECK(void, exit_fn, void*);\
    TC_FN_TYPECHECK(void, update_fn, void*, float);\
    TC_FN_TYPECHECK(const State*, next_fn, void*);\
    static const State instance = {\
        .enter = (void(*const)(void*)) enter_fn,\
        .exit = (void(*const)(void*)) exit_fn,\
        .update = (void(*const)(void*, float)) update_fn,\
        .next = (const State*(*const)(void*)) next_fn\
    };\
    return instance;\
}\

#endif // !_fencer_state_h
